"use strict";

this.name           = "TargetSystemPlugins_FilterSort";
this.author         = "Nicholas Menchise";
this.copyright      = "(C) 2019 Nicholas Menchise";
this.licence        = "CC BY-NC-SA 4.0";

//
// This world script includes the instructions for filtering entities within
// scanner range and sorting them into groups for the equipment.
//


// Total targets within scanner range
this.$targetsTotal = [];

// Hostile targets within scanner range
this.$targetsHostile = [];

// Outlaw targets within scanner range
this.$targetsOutlaw = [];

// Neutral targets within scanner range
this.$targetsNeutral = [];

// State targets within scanner range
this.$targetsState = [];

// Cargo targets within scanner range
this.$targetsCargo = [];

// Rock targets within scanner range
this.$targetsRock = [];

// Other targets within scanner range
this.$targetsOther = [];

// Memory bank of Entities for the TMI MFD.
this.$targetsMemoryData = [];

// Target category symbol strings. There should be a symbol for each Entity.
this.$targetsMemorySigns = [];

// Memory bank index positions of valid Entities within player ship scanner range, sorted by distance.
this.$targetsMemoryInRange = [];


//
// Sets the array of total targets within scanner range.
//
// This function is called by _updateTallies function in world script TargetSystemPlugins_ShortRangeSnapshot.
//
// The predicate for the filteredEntities method of System class excludes
// RRS autopilot screens for better compatibility with the Rescue Stations expansion.
//
this._getTargetsTotal = function() {
    this.$targetsTotal = system.filteredEntities(
        this,
        function(e) {
            if (
                e.isShip && !e.isCloaked && e.displayName !== "Wreckage" && e.dataKey !== "rrs_autopilot_screen"
            ) return e;
        },
        player.ship, player.ship.scannerRange
    );
}


//
// Sets the arrays of targets in seven categories.
//
// This function is called by _updateTallies function in world script TargetSystemPlugins_ShortRangeSnapshot.
//
this._getTargetsSorted = function() {

    // Hostile mode
    this.$targetsHostile = this._getTargetsFiltered(
        function(e) {
            if (e.hasHostileTarget && e.target === player.ship) {
                return true;
            }
                else {
                    return false;
                }
        }
    );

    // Outlaw mode
    this.$targetsOutlaw = this._getTargetsFiltered(
        function(e) {
            if (e.scanClass === "CLASS_THARGOID" || e.scanClass === "CLASS_NEUTRAL" && e.bounty !== 0) {
                return true;
            }
                else {
                    return false;
                }
        }
    );

    // Neutral mode, includes additional parameters for better compatibility with the Escort Contracts expansion
    this.$targetsNeutral = this._getTargetsFiltered(
        function(e) {
            if (e.beaconLabel === "Escort Mother" || e.scanClass === "CLASS_NEUTRAL" && e.bounty === 0) {
                return true;
            }
                else {
                    return false;
                }
        }
    );

    // State mode
    this.$targetsState = this._getTargetsFiltered(
        function(e) {
            if (e.scanClass === "CLASS_POLICE" || e.scanClass === "CLASS_MILITARY") {
                return true;
            }
                else {
                    return false;
                }
        }
    );

    // Cargo mode, includes additional parameters for better compatibility with the Ore Processor expansion
    this.$targetsCargo = this._getTargetsFiltered(
        function(e) {
            if (e.scanClass === "CLASS_CARGO" && e.displayName !== "Splinter") {
                return true;
            }
                else {
                    return false;
                }
        }
    );

    // Rock mode, includes additional parameters for better compatibility with Ore Processor and Escort Contracts expansions
    this.$targetsRock = this._getTargetsFiltered(
        function(e) {
            if (e.displayName === "Splinter" || e.scanClass === "CLASS_ROCK" && e.beaconLabel !== "Escort Mother") {
                return true;
            }
                else {
                    return false;
                }
        }
    );

    // Other mode
    this.$targetsOther = this._getTargetsFiltered(
        function(e) {
            if (
                e.scanClass !== "CLASS_THARGOID" && e.scanClass !== "CLASS_NEUTRAL" && e.scanClass !== "CLASS_POLICE" &&
                e.scanClass !== "CLASS_MILITARY" && e.scanClass !== "CLASS_CARGO" && e.scanClass !== "CLASS_ROCK"
            ) {
                return true;
            }
                else {
                    return false;
                }
        }
    );

}


//
// Returns an array of Entities that matches a target category.
//
// This function is called by _getTargetsSorted function.
//
// filterThisWay argument is a function with one argument variable and one condition that returns a boolean.
// The condition includes the filter parameters. It should return true within the if statement and false within the else statement.
//
this._getTargetsFiltered = function(filterThisWay) {
    var giveTheseTargets = [];

    for (var n = 0; n < this.$targetsTotal.length; n++) {
        var tryThisOne = filterThisWay(this.$targetsTotal[n]);
        if (tryThisOne) var x = giveTheseTargets.push(this.$targetsTotal[n]);
    }

    return giveTheseTargets;
}


//
// Returns the number of Entities in a target category that matches a parameter.
//
// This function is called by _updateTallies function in world script TargetSystemPlugins_ShortRangeSnapshot.
//
// This should only be called on the condition that there are any targets in the relevant category.
//
// whichTally argument is a string that specifies the targets to be counted.
// There are three valid strings for this argument:
// "thargoid" for Thargoid ships
// "escape" for escape capsules
// "splinter" for rock splinters.
//
// Returns a negative integer if the string is invalid or the array to be searched is empty.
//
this._getTally = function(whichTally) {
    var tallyResult = -1;
    var checkList = [];

    // Determines the target array to be searched.
    if (whichTally === "thargoid") checkList = this.$targetsOutlaw;
    if (whichTally === "escape") checkList = this.$targetsCargo;
    if (whichTally === "splinter") checkList = this.$targetsRock;

    if (checkList.length < 1) return tallyResult;
    tallyResult = 0;

    // Determines the filter function to be applied to the target array.
    var countThese = function(){};
    if (whichTally === "thargoid")
        countThese = function(e) {
            if (e.scanClass === "CLASS_THARGOID") {
                return true;
            }
                else {
                    return false;
                }
        };
    if (whichTally === "escape")
        countThese = function(e) {
            if (e.hasRole("escape-capsule")) {
                return true;
            }
                else {
                    return false;
                }
        };
    if (whichTally === "splinter")
        countThese = function(e) {
            if (e.displayName === "Splinter") {
                return true;
            }
                else {
                    return false;
                }
        };

    // Searches the target array and increments the tally when the correct entries are found.
    for (var n = 0; n < checkList.length; n++) {
        var tryThisOne = countThese(checkList[n]);
        if (tryThisOne) tallyResult++;
    }

    return tallyResult;
}


//
// Tests an Entity to determine if it goes into the target memory bank and does so if it passes.
//
// This function is called by shipTargetAcquired event handler in world script TargetSystemPlugins_SharedHandlers.
//
// testThis argument is the Entity to be tested.
//
this._tmiBankThisOrNot = function(testThis) {

    // Checks if the Entity is already in the memory bank to avoid duplication.
    var matchFound = this._tmiCheckForMatch(testThis, this.$targetsMemoryData);

    // Clears invalid entries and keeps the memory bank array length to 8 or less.
    this._tmiCleanUp();
    if (!matchFound && this.$targetsMemoryData.length == 8) var removeD = this.$targetsMemoryData.shift();
    if (!matchFound && this.$targetsMemorySigns.length == 8) var removeS = this.$targetsMemorySigns.shift();

    if (!matchFound) {
        // Adds target information to the end of the arrays.
        var x = this.$targetsMemoryData.push(testThis);
        var srsWorking = worldScripts[expandDescription("[tsplugins_script_srs]")]._statusCheck();
        var signOfThis = "";
        if (srsWorking) signOfThis = this._tmiSignThis(testThis);
        if (!srsWorking) signOfThis = expandDescription("[tsplugins_sign_missed]");    // symbol for unknown
        x = this.$targetsMemorySigns.push(signOfThis);
    }
        else {
            // Transfers the target information from its current position in the arrays to the end of the arrays.
            var whereIsThis = this.$targetsMemoryData.indexOf(testThis);
            var transferD = this.$targetsMemoryData[whereIsThis];
            var transferS = this.$targetsMemorySigns[whereIsThis];
            removeD = this.$targetsMemoryData.splice(whereIsThis, 1);
            removeS = this.$targetsMemorySigns.splice(whereIsThis, 1);
            var y = this.$targetsMemoryData.push(transferD);
            y = this.$targetsMemorySigns.push(transferS);
        }

}


//
// Tests the target memory bank for invalid entries and removes them, if so.
//
// This function is called by _tmiBankThisOrNot function.
//
this._tmiCleanUp = function() {
    for (var n = 0; n < this.$targetsMemoryData.length; n++) {
        if (!this.$targetsMemoryData[n].isInSpace) {
            var removeD = this.$targetsMemoryData.splice(n, 1);
            var removeS = this.$targetsMemorySigns.splice(n, 1);
            n--;
        } else {}
    }
}


//
// Sets an array of index positions of $targetsMemoryData entries within the player ship's scanner range, sorted
// by distance from the player ship. The array is assigned to $targetsMemoryInRange.
// Also moves targets in memory to the front of their arrays if they're out of scanner range, which puts them first
// in line for removal by _tmiBankThisOrNot function when trimming the length.
//
// This function is called by _timedUpdate function in world script TargetSystemPlugins_TargetMemoryInterface.
// It is also called by shipTargetAcquired event handler in world script TargetSystemPlugins_SharedHandlers.
// It is also called by activated event handler in equipment script TargetSystemPlugins_TargetBankingModule.
//
this._tmiGroupInRange = function() {

    // Gets the index positions of targets within scanner range and their distances from player ship.
    var youScanThem = [];
    var youMeasureThem = [];
    var youAreHere = player.ship.position;
    var outThere = player.ship.scannerRange;
    for (var n = 0; n < this.$targetsMemoryData.length; n++) {
        var checkThisOne = this.$targetsMemoryData[n];
        if (!checkThisOne.isInSpace || checkThisOne.isCloaked) {
        }
            else if (youAreHere.distanceTo(checkThisOne) < outThere) {
                var foundThisOne = youAreHere.distanceTo(checkThisOne);
                var x = youScanThem.push(n);
                x = youMeasureThem.push(foundThisOne);
            }
                else {
                    // Transfers the target information from its current position in the arrays to the start of the arrays.
                    var transferD = this.$targetsMemoryData[n];
                    var transferS = this.$targetsMemorySigns[n];
                    var removeD = this.$targetsMemoryData.splice(n, 1);
                    var removeS = this.$targetsMemorySigns.splice(n, 1);
                    x = this.$targetsMemoryData.unshift(transferD);
                    x = this.$targetsMemorySigns.unshift(transferS);
                }
    }

    // Sorts the entries by distance from player ship.
    var youSortThem = [];
    var howMany = youScanThem.length;
    for (var n = 0; n < howMany; n++) {
        var closestRange = outThere;
        var closestOne = -1;
        for (var p = 0; p < youMeasureThem.length; p++) {
            if (youMeasureThem[p] < closestRange) closestRange = youMeasureThem[p];
            if (youMeasureThem[p] == closestRange) closestOne = p;
        }
        var y = youSortThem.push(youScanThem[closestOne]);
        var removeN = youScanThem.splice(closestOne, 1);
        var removeM = youMeasureThem.splice(closestOne, 1);
    }

    this.$targetsMemoryInRange = youSortThem;
}


//
// Initialises the TMI arrays.
//
// This function is called by startUp, shipWillDockWithStation, shipWillEnterWitchspace, equipmentDamaged and
// equipmentRemoved event handlers in world script TargetSystemPlugins_SharedHandlers.
//
this._tmiReset = function() {
    this.$targetsMemoryData = [];
    this.$targetsMemorySigns = [];
    this.$targetsMemoryInRange = [];
}


//
// Tests the Entities in the target memory bank to determine if their TMI category symbols are current and changes
// them, if necessary.
//
// This function is called by _timedUpdate function in world script TargetSystemPlugins_TargetMemoryInterface.
//
// This should only be called if the SRS equipment is functional.
//
this._tmiSignUpdate = function() {
    var signRecorded = "";    // stores the TMI category symbol that's in the memory bank
    var signScanned = "";    // stores the TMI category symbol that's in the latest snapshot, if target is found

    for (var n = 0; n < this.$targetsMemorySigns.length; n++) {
        signRecorded = this.$targetsMemorySigns[n];
        if (!this.$targetsMemoryData[n].isInSpace || this.$targetsMemoryData[n].isCloaked) {
            signScanned = expandDescription("[tsplugins_sign_missed]");
        }
            else {
                signScanned = this._tmiSignThis(this.$targetsMemoryData[n]);
            }
        if (signRecorded !== signScanned) this.$targetsMemorySigns[n] = signScanned;
    }
}


//
// Tests an Entity to determine its target category and returns its TMI symbol string.
//
// This function is called by _tmiBankThisOrNot and _tmiSignUpdate functions.
//
// This should only be called if the SRS equipment is functional.
//
// whatsThat argument is the Entity to be tested.
//
this._tmiSignThis = function(whatsThat) {
    var signNotFound = expandDescription("[tsplugins_sign_missed]");    // symbol for unknown

    // Compares the Entity to the Entities in seven arrays until match found or searched all
    var matchFound = false;
    var checkThese = [
        this.$targetsHostile, this.$targetsOutlaw,
        this.$targetsNeutral, this.$targetsState,
        this.$targetsCargo, this.$targetsRock,
        this.$targetsOther
    ];
    var signWithThis = [
        expandDescription("[tsplugins_sign_mode0]"), expandDescription("[tsplugins_sign_mode1]"),
        expandDescription("[tsplugins_sign_mode2]"), expandDescription("[tsplugins_sign_mode3]"),
        expandDescription("[tsplugins_sign_mode4]"), expandDescription("[tsplugins_sign_mode5]"),
        expandDescription("[tsplugins_sign_mode6]")
    ];
    for (var n = 0; n < checkThese.length; n++) {
        matchFound = this._tmiCheckForMatch(whatsThat, checkThese[n]);
        if (matchFound) return signWithThis[n];
    }

    return signNotFound;
}


//
// Returns a boolean that indicates if an Entity matches an entry in an array of Entities.
//
// This function is called by _tmiBankThisOrNot and _tmiSignThis functions.
//
// checkThis argument is the Entity to be tested.
// againstThis argument is the array of Entities.
//
this._tmiCheckForMatch = function(checkThis, againstThis) {

    for (var n = 0; n < againstThis.length; n++) {
        if (checkThis === againstThis[n]) return true;
    }

    return false;
}

