"use strict";

this.name           = "TargetSystemPlugins_ShortRangeSnapshot";
this.author         = "Nicholas Menchise";
this.copyright      = "(C) 2019 Nicholas Menchise";
this.licence        = "CC BY-NC-SA 4.0";

//
// This world script includes the instructions for the SRS MFD strings and checking the equipment status.
//


// Whether or not the equipment is on the player ship
this.$onShip = false;

// Whether or not the equipment is functioning
this.$ready = false;

// Whether or not MFD updates should be suspended to prevent tip strings from changing every second
this.$mfdSuspend = false;

// Calls the function that updates the equipment MFD every second, if the equipment is functioning
this.$readyUpdate = new Timer(this, function() { this._timedUpdate(); }, 0, 1);

// Stores the MFD key for the equipment
this.$mfdKey = "tsplugins_srs";

// Stores the equipment's target tallies from the last timed update
//
// This is stored as a global variable so that the target mode selection can be updated independently on the MFD.
// The tallies should be stored in the following order:
// Hostile, Outlaw, Neutral, State, Cargo, Rock, Other, Total, Thargoids, Escape Capsules, Splinters.
//
this.$tallies = [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0];


//
// Checks the status of the equipment and its dependencies. Returns a boolean.
//
// This function is called by _timedUpdate function. It is also called by _statusCheck function in equipment
// script TargetSystemPlugins_TargetBankingModule. It is also called by event handlers in world script
// TargetSystemPlugins_SharedHandlers. It is also called by _cascadeWeaponAlert function in world script
// TargetSystemPlugins_SharedHandlers.
//
// A negative status (false) will trigger a function call that results in a status update.
// An integer is calculated and passed in the same function call to provide more specific
// information in the status update.
//
this._statusCheck = function() {
    var dependsValue = 0;
    var dependsResult = false;

    if (player.ship.equipmentStatus(expandDescription("[tsplugins_eqkey_srs]")) === "EQUIPMENT_OK") dependsValue = 2;
    if (player.ship.equipmentStatus("EQ_SCANNER_SHOW_MISSILE_TARGET") === "EQUIPMENT_OK") dependsValue++;

    if (dependsValue == 3) dependsResult = true;
    if (this.$ready && dependsValue < 3) this._statusUpdate(false, dependsValue);

    return dependsResult;
}


//
// Updates the status of the equipment and informs the player when it's activated, disabled or damaged.
//
// This function is called by _statusCheck function. It is also called by event handlers in the world
// script TargetSystemPlugins_SharedHandlers.
//
// isWorking argument stores a boolean that indicates if the equipment is enabled.
//
// whatHappened argument stores an integer that indicates the cause of the status change.
// A value of 3 means that the equipment is fine, but something else may be stopping it, e.g. witchspace tunnel.
// A value of 2 means that the dependency check has failed.
// A value less than 2 means that the equipment is damaged.
//
this._statusUpdate = function(isWorking, whatHappened) {

    // Changes the ready status and stops the timer, if necessary. Stops the function if equipment is off the ship.
    if (this.$ready !== isWorking) this.$ready = isWorking;
    if (!isWorking && this.$readyUpdate.isRunning) this.$readyUpdate.stop();
    if (!this.$onShip) return;

    // Sets the string for the status update to the MFD.
    var theseLines = "";    // stores the MFD update string.
    if (isWorking) theseLines = expandDescription("[tsplugins_ready_srs]");
    if (!isWorking && whatHappened == 3) theseLines = expandDescription("[tsplugins_title_srs][tsplugins_mfd_status_witchspace]");
    if (!isWorking && whatHappened == 2) theseLines = expandDescription("[tsplugins_disabled1_srs]");
    if (!isWorking && whatHappened < 2) theseLines = expandDescription("[tsplugins_broken_srs]");

    // Calls the MFD setting function.
    worldScripts[expandDescription("[tsplugins_script_sh]")]._setMfd(this.$mfdKey, theseLines);

    // Resumes the timer that calls regular updates to the MFD, if the equipment has become functional.
    // The two second delay ensures that the player can see the status update when the equipment is activated.
    //
    if (isWorking && !this.$readyUpdate.isRunning) {
        this.$readyUpdate.nextTime = clock.absoluteSeconds + 2;
        this.$readyUpdate.start();
        this.$mfdSuspend = false;    // fixed a bug where the MFD could get stuck on startup instead of displaying tip
    }
        else {
        }
}


//
// Updates the equipment MFD at regular intervals, if the equipment is functional.
//
// This function is called by $readyUpdate timer.
//
this._timedUpdate = function() {
    var isWorking = this._statusCheck();
    if (!isWorking) return;
    this._updateTallies();
    if (this.$tallies[7] > 0) this.$mfdSuspend = false;
    if (!this.$mfdSuspend) this._setMfdString();
}


//
// Calculates and sets the integers in the tallies array.
//
// This function is called by _timedUpdate function.
//
this._updateTallies = function() {

    // Sets the entries for additional tallies to zero to prevent outdated values from showing up.
    for (var n = 8; n < this.$tallies.length; n++) this.$tallies[n] = 0;

    // Updates the target data within the player ship's scanner range.
    var tspFS = worldScripts[expandDescription("[tsplugins_script_fs]")];
    tspFS._getTargetsTotal();
    tspFS._getTargetsSorted();

    // Sets the tallies for the target modes.
    this.$tallies[0] = tspFS.$targetsHostile.length;
    this.$tallies[1] = tspFS.$targetsOutlaw.length;
    this.$tallies[2] = tspFS.$targetsNeutral.length;
    this.$tallies[3] = tspFS.$targetsState.length;
    this.$tallies[4] = tspFS.$targetsCargo.length;
    this.$tallies[5] = tspFS.$targetsRock.length;
    this.$tallies[6] = tspFS.$targetsOther.length;
    this.$tallies[7] = tspFS.$targetsTotal.length;

    // Sets additional tallies, if applicable.
    if (this.$tallies[1] > 0) this.$tallies[8] = tspFS._getTally("thargoid");
    if (this.$tallies[4] > 0) this.$tallies[9] = tspFS._getTally("escape");
    if (this.$tallies[5] > 0) this.$tallies[10] = tspFS._getTally("splinter");

}


//
// Sets the string that will be sent to the equipment MFD and calls the MFD update.
//
// This function is called by _timedUpdate function. It is also called by event handlers in equipment
// script TargetSystemPlugins_TargetBankingModule. It is also called by playerChangedPrimedEquipment
// event handler in world script TargetSystemPlugins_SharedHandlers.
//
this._setMfdString = function() {
    var sayThisMfd = "";
    var tspSH = worldScripts[expandDescription("[tsplugins_script_sh]")];

    // Sets the snapshot string when targets are found.
    if (this.$tallies[7] > 0) {

        // Sets the first line, which includes the title and timestamp. The second line is blank.
        var snapshotTime = "";
        if (clock.isAdjusting) snapshotTime = "Ad:ju:st";
        if (!clock.isAdjusting) snapshotTime = clock.clockStringForTime(clock.adjustedSeconds).substring(8);
        var lineOne = expandDescription("[tsplugins_title_srs]");
        lineOne += "        " + snapshotTime + "\n" + "\n";

        // Sets the target mode selection column.
        var modeColumn = [];
        for (var n = 0; n < 8; n++) modeColumn[n] = expandDescription("[tsplugins_mfd_unselected]");
        var tmiWorking = worldScripts[expandDescription("[tsplugins_script_tmi]")]._statusCheck();
        if (
            tmiWorking && tspSH.$tbmReady && this.$tallies[tspSH.$tbmTargetMode] > 0 &&
            player.ship.primedEquipment === expandDescription("[tsplugins_eqkey_tbm]")
        ) modeColumn[tspSH.$tbmTargetMode] = expandDescription("[tsplugins_mfd_selected]");

        // Sets the target tally lines and puts them next to the target mode selection column.
        var tallyText = this._getTallyList();
        var linesEight = "";
        for (var n = 0; n < modeColumn.length; n++) {
            linesEight = linesEight + modeColumn[n] + tallyText[n];
            if (n < 7) linesEight += "\n";
        }

        sayThisMfd = lineOne + linesEight;
    }
        else {
            // Sets the SRS Tip string when no targets are found.
            sayThisMfd = expandDescription("[tsplugins_mfd_tip_srs]");
            this.$mfdSuspend = true;
        }

    // Calls the MFD setting function.
    tspSH._setMfd(this.$mfdKey, sayThisMfd);
}


//
// Returns an array of strings for the target tally text.
//
// This function is called by _setMfdString function.
//
this._getTallyList = function() {
    var tallyList = ["", "", "", "", "", "", "", ""];
    var tallyLabels = [
        expandDescription("[tsplugins_title_mode0]"), expandDescription("[tsplugins_title_mode1]"),
        expandDescription("[tsplugins_title_mode2]"), expandDescription("[tsplugins_title_mode3]"),
        expandDescription("[tsplugins_title_mode4]"), expandDescription("[tsplugins_title_mode5]"),
        expandDescription("[tsplugins_title_mode6]"), expandDescription("[tsplugins_title_mode7]")
    ];

    // Sets the left column of the tally text. Tallies with no targets are not displayed.
    for (var n = 0; n < tallyList.length; n++) {
        if (this.$tallies[n] > 0) tallyList[n] = tallyLabels[n] + ": " + this.$tallies[n];
    }

    // Appends additional tallies, if applicable.
    if (this.$tallies[8] > 0) tallyList[1] += expandDescription("[tsplugins_mfd_gap]") + this.$tallies[8] + " Thargoid ship";
    if (this.$tallies[8] > 1) tallyList[1] += "s";
    if (this.$tallies[9] > 0) tallyList[4] += expandDescription("[tsplugins_mfd_gap]") + this.$tallies[9] + " escape pod";
    if (this.$tallies[9] > 1) tallyList[4] += "s";
    if (this.$tallies[10] > 0) tallyList[5] += expandDescription("[tsplugins_mfd_gap]") + this.$tallies[10] + " splinter";
    if (this.$tallies[10] > 1) tallyList[5] += "s";

    return tallyList;
}

