"use strict";

this.name           = "TargetSystemPlugins_TargetBankingModule";
this.author         = "Nicholas Menchise";
this.copyright      = "(C) 2019 Nicholas Menchise";
this.licence        = "CC BY-NC-SA 4.0";

//
// This equipment script includes the instructions for the TBM's "mode" and
// "activated" event handlers and their support functions, e.g. the status check.
//



// Stores an integer that symbolises the state of the equipment, which enables the HUD
// console messages to provide more specific explanations when the controls are disabled.
//
// There are four valid integer values. The values are set by _statusCheck function.
// Zero means that a dependency was unavailable on the last two status checks.
// One means that dependencies are available, but the equipment is temporarily disabled by a witchspace tunnel.
// Two means that a dependency was unavailable on the last status check.
// Three means that all equipment is fine, but the controls could still be disabled if there are no targets in SRS.
//
this.$statusInfo = 0;


//
// Checks the status of the equipment and its dependencies. Returns a boolean.
//
// This function is called by the .mode and .activated event handlers.
//
// An integer is calculated and set to the status info variable. See the variable comments for details.
//
// The equipment's ready status variable in world script TargetSystemPlugins_SharedHandlers is set to false if a
// dependency is unavailable.
//
this._statusCheck = function() {
    var dependsValue = 0;
    var dependsResult = false;
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
    var tspSH = worldScripts[expandDescription("[tsplugins_script_sh]")];
    var srsWorking = tspSRS._statusCheck();
    var tmiWorking = worldScripts[expandDescription("[tsplugins_script_tmi]")]._statusCheck();
    var tbmWorking = tspSH.$tbmReady;

    // The condition below refers to the tbmReady variable instead of the equipmentStatus method of PlayerShip object,
    // because it applies to situations where the equipment is temporarily disabled. The equipment script does nothing
    // if the equipment is damaged, because it's not primeable.
    //
    if (tbmWorking) dependsValue = 2;
    if (srsWorking && tmiWorking) dependsValue++;

    if (dependsValue == 3) dependsResult = true;
    if (tbmWorking && dependsValue < 3) tspSH.$tbmReady = false;

    // Disables the equipment controls if no targets are in SRS arrays.
    // The tbmReady variable is not changed, because it's unnecessary.
    //
    if (tspSRS.$tallies[7] < 1) dependsResult = false;

    this.$statusInfo = dependsValue;
    return dependsResult;
}


//
// Informs the player that the equipment is disabled and why. The message is sent to the HUD console.
//
// This function is called by the .mode and .activated event handlers.
//
// This should only be called after a status check returns a negative value.
//
this._statusMessage = function() {
    var sendThis = expandDescription("[tsplugins_console_tbm_disabled_alert] ");
    if (this.$statusInfo == 1) sendThis += expandDescription("[tsplugins_console_tbm_disabled_why1]");
    if (this.$statusInfo == 2 || this.$statusInfo == 0) sendThis += expandDescription("[tsplugins_console_tbm_disabled_why2]");
    if (this.$statusInfo == 3) sendThis += expandDescription("[tsplugins_console_tbm_disabled_why3]");
    player.consoleMessage(sendThis);
}


//
// Returns the correct array of Entities for target banking.
//
// This function is called by .activated event handler.
//
// targetMode argument is the equipment's current target mode.
//
this._bankTheseTargets = function(targetMode) {
    var theseTargets = [];
    var tspFS = worldScripts[expandDescription("[tsplugins_script_fs]")];
    if (targetMode == 0) theseTargets = tspFS.$targetsHostile;
    if (targetMode == 1) theseTargets = tspFS.$targetsOutlaw;
    if (targetMode == 2) theseTargets = tspFS.$targetsNeutral;
    if (targetMode == 3) theseTargets = tspFS.$targetsState;
    if (targetMode == 4) theseTargets = tspFS.$targetsCargo;
    if (targetMode == 5) theseTargets = tspFS.$targetsRock;
    if (targetMode == 6) theseTargets = tspFS.$targetsOther;
    if (targetMode == 7) theseTargets = tspFS.$targetsTotal;
    return theseTargets;
}


//
// Event handlers. Specified on Elite wiki page Oolite Javascript Reference: Equipment scripts.
//


this.mode = function() {

    // Checks if the equipment is working.
    var tbmWorking = this._statusCheck();

    if (tbmWorking) {

        // Finds and sets the next valid target mode.
        var tspSH = worldScripts[expandDescription("[tsplugins_script_sh]")];
        var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];
        tspSH.$tbmTargetMode++;
        if (tspSH.$tbmTargetMode > 7) tspSH.$tbmTargetMode = 0;
        for (var n = tspSH.$tbmTargetMode; n < 8; n++) {
            if (tspSRS.$tallies[n] > 0) break;
            tspSH.$tbmTargetMode++;
        }

        // Updates the SRS MFD with the next valid target mode.
        tspSRS._setMfdString();

    }
        else {
            // Informs the player that the equipment is not working.
            this._statusMessage();
        }

}


this.activated = function() {

    var tspSH = worldScripts[expandDescription("[tsplugins_script_sh]")];
    var tspSRS = worldScripts[expandDescription("[tsplugins_script_srs]")];

    // Checks if the equipment is working.
    var tbmWorking = this._statusCheck();

    if (!tbmWorking) {
        // Informs the player that the equipment is not working.
        this._statusMessage();
    }
        else if (tspSRS.$tallies[tspSH.$tbmTargetMode] < 1) {
            // Informs the player that there are no targets in the selected mode and switches to Total mode.
            player.consoleMessage(expandDescription("[tsplugins_console_tbm_none_mode]"));
            tspSH.$tbmTargetMode = 7;
            tspSRS._setMfdString();
        }
            else {

                // Assigns targets in a valid mode to player ship.
                // The maximum number of targets is 8, because that's the limit of the TMI MFD.
                // The loop works backwards so that the last new target is the nearest one to the player ship.
                // The loop checks if the target is still around in case something happened to it after the last SRS update.
                // The loop avoids passing targets already in TMI memory, because doing so would be a waste of cycles.
                // The tbmActive state prevents the shipTargetAcquired event handler from updating the TMI MFD.
                //
                tspSH.$tbmActive = true;
                var tspFS = worldScripts[expandDescription("[tsplugins_script_fs]")];
                var howMany = tspSRS.$tallies[tspSH.$tbmTargetMode];
                if (howMany > 8) howMany = 8;
                var whichTargets = this._bankTheseTargets(tspSH.$tbmTargetMode);
                for (var n = 1; n <= howMany; n++) {
                    var thisTarget = whichTargets[howMany - n];
                    var matchFound = tspFS._tmiCheckForMatch(thisTarget, tspFS.$targetsMemoryData);
                    if (thisTarget.isInSpace && !matchFound) player.ship.target = thisTarget;
                }

                // Updates the TMI MFD.
                tspFS._tmiGroupInRange();
                worldScripts[expandDescription("[tsplugins_script_tmi]")]._setMfdString();
                tspSH.$tbmActive = false;

            }

}

